//
//  UIImageViewItemView.m
//  room
//
//  Created by  on 12-7-16.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "UIImageViewItemView.h"

@implementation UIImageViewItemView
@synthesize delegate = _delegate;
@synthesize isTransformGreaterThanZero = _isTransformGreaterThanZero;
@synthesize imageName       = _imageName;
@synthesize pickViewCoin    = _pickViewCoin;
@synthesize isHighLighted   = _isHighLighted;
@synthesize pickUniqueNumber = _pickUniqueNumber;

- (id) init{
    self = [super init];
    if (self) {
        self.userInteractionEnabled = YES;
        self.imageName = @"0";
    }
    return self;
}

//设置UIView是否开启或者关闭多手势识别功能
- (void) setupHandGestureRecognizerWithDraggable:(BOOL)isDraggable 
                                   withPinchable:(BOOL)isPinchable 
                                  withRotateable:(BOOL)isRotateable
                                     withTapable:(BOOL)isTapable 
                               withMinPinchScale:(CGFloat)minPScale 
                               withMaxPinchScale:(CGFloat)maxPScale
{
    _isTransformGreaterThanZero = YES;
    
    //监测是否关闭多手势识别
    NSMutableDictionary *dict = (NSMutableDictionary*) (void*) self.tag;
    if (!(isDraggable || isPinchable || isRotateable)) {
        if (dict) {
            [dict removeAllObjects];
            self.tag = (NSInteger) (void *) dict;
        }
        for(UIGestureRecognizer *gesture in [self gestureRecognizers]){
            [self removeGestureRecognizer:gesture];
        }
        return;
    }
    //清空识别器
    if (dict) {
        UIPanGestureRecognizer *pan =[dict objectForKey:@"UIPanGestureRecognizer"];
        if(pan){
            if ([self.gestureRecognizers indexOfObject:pan] != NSNotFound) {
                [self removeGestureRecognizer:pan];
            }
            [dict removeObjectForKey:@"UIPanGestureRecognizer"];
        }
        
        UIPinchGestureRecognizer *pinch =[dict objectForKey:@"UIPinchGestureRecognizer"];
        if(pinch){
            if ([self.gestureRecognizers indexOfObject:pinch] != NSNotFound) {
                [self removeGestureRecognizer:pinch];
            }
            [dict removeObjectForKey:@"UIPinchGestureRecognizer"];
        }
        
        UIPinchGestureRecognizer *rotationGesture =[dict objectForKey:@"UIPinchGestureRecognizer"];
        if(rotationGesture){
            if ([self.gestureRecognizers indexOfObject:pinch] != NSNotFound) {
                [self removeGestureRecognizer:pinch];
            }
            [dict removeObjectForKey:@"UIPinchGestureRecognizer"];
        }
        
        UITapGestureRecognizer  *tapGesture = [dict objectForKey:@"UITapGestureRecognizer"];
        if (tapGesture) {
            if ([self.gestureRecognizers indexOfObject:tapGesture] != NSNotFound) {
                [self removeGestureRecognizer:tapGesture];
            }
            [dict removeObjectForKey:@"UITapGestureRecognizer"];
        }
        [dict removeObjectForKey:@"startDragLocation"];
        [dict removeObjectForKey:@"startDragOffset"];
        [dict removeObjectForKey:@"lastScale"];
        [dict removeObjectForKey:@"minScale"];
        [dict removeObjectForKey:@"maxScale"];
    }
    //根据设置重置识别器
    if (isDraggable) {
        UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(fingerDidMoveInDraggableView:)];
        pan.minimumNumberOfTouches = 1;  
        pan.maximumNumberOfTouches = 1;  
        [self addGestureRecognizer:pan];
        [pan release];
        
        dict = [self dictForViewGuestures];
        [dict setObject:pan forKey:@"UIPanGestureRecognizer"];
    }
    
    if (isPinchable) {
        //CGAffineTransform initialTramsform = CGAffineTransformMakeScale(1.0, 1.0);
        //self.transform = initialTramsform;
        
        
        UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(fingersDidPinchInPinchableView:)];
        [self addGestureRecognizer:pinch];
        [pinch release];
        dict = [self dictForViewGuestures];
        [dict setObject:pinch forKey:@"UIPinchGestureRecognizer"];
        [dict setObject:[NSNumber numberWithFloat:minPScale] forKey:@"minScale"];
        [dict setObject:[NSNumber numberWithFloat:maxPScale] forKey:@"maxScale"];
        
    }
    
    if(isRotateable){
        UIRotationGestureRecognizer *rotationGesture = [[UIRotationGestureRecognizer alloc] initWithTarget:self action:@selector(fingersDidRotateInRotateableView:)];
        [self addGestureRecognizer:rotationGesture];
        [rotationGesture release];    
        dict = [self dictForViewGuestures];
        [dict setObject:rotationGesture forKey:@"UIRotationGestureRecognizer"];
        
    }
    
    if (isTapable) {
        UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(fingerDidTapInTapableView:)];
        [self addGestureRecognizer:tapGesture];
        [tapGesture release];
        dict = [self dictForViewGuestures];
        [dict setObject:tapGesture forKey:@"UITapGestureRecognizer"];
    }
}

- (void)fingerDidTapInTapableView:(UITapGestureRecognizer *)tapRecognizer{
    [self.delegate setControllPoint:self];
}

- (void)fingersDidRotateInRotateableView:(UIRotationGestureRecognizer *)gestureRecognizer{
    UIView *piece = gestureRecognizer.view;
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        CGPoint locationInView = [gestureRecognizer locationInView:piece];
        CGPoint locationInSuperview = [gestureRecognizer locationInView:piece.superview];        
        piece.layer.anchorPoint = CGPointMake(locationInView.x / piece.bounds.size.width, locationInView.y / piece.bounds.size.height);
        piece.center = locationInSuperview;
    }
    if ([gestureRecognizer state] == UIGestureRecognizerStateBegan || [gestureRecognizer state] == UIGestureRecognizerStateChanged){    
        piece.transform = CGAffineTransformRotate([piece transform], [gestureRecognizer rotation]);
        [gestureRecognizer setRotation:0];
    } 
}

- (void)fingersDidPinchInPinchableView:(UIPinchGestureRecognizer *)fingers {
    NSMutableDictionary *dict = [self dictForViewGuestures];
    CGFloat lastScale;
    CGFloat tarScale;
    if([fingers state] == UIGestureRecognizerStateBegan) {
        lastScale = [fingers scale];
    } else {
        lastScale = [[dict objectForKey:@"lastScale"] floatValue];
    }
    
    if ([fingers state] == UIGestureRecognizerStateBegan || 
        [fingers state] == UIGestureRecognizerStateChanged) {
        
        CGFloat currentScale = [[self.layer valueForKeyPath:@"transform.scale"] floatValue];
        //如果图像已经进行了左旋操作
        if (_isTransformGreaterThanZero == YES) {
            tarScale = currentScale;
        }else{
            tarScale = 0-currentScale;
        }
        
        CGFloat maxScale = [[dict objectForKey:@"maxScale"] floatValue];
        CGFloat minScale = [[dict objectForKey:@"minScale"] floatValue];
        
        CGFloat newScale = 1 -  (lastScale - [fingers scale]); 
        newScale = MIN(newScale, maxScale / tarScale);   
        newScale = MAX(newScale, minScale / tarScale);
        
        CGAffineTransform transform = CGAffineTransformScale([self transform], newScale, newScale);
        self.transform = transform;
        
        lastScale = [fingers scale]; 
    }
    
    [dict setObject:[NSNumber numberWithFloat:lastScale] 
             forKey:@"lastScale"];
}

- (void)fingerDidMoveInDraggableView:(UIPanGestureRecognizer *)finger{
    NSMutableDictionary *dict = [self dictForViewGuestures];  
    [self.delegate setControllPoint:self];
    //[self.delegate isPickViewHalfHide];
    
    if (finger.state == UIGestureRecognizerStateBegan) {
        
        
        [dict setObject:[NSValue valueWithCGPoint:self.frame.origin ]
                 forKey:@"startDragOffset" ];          
        [dict setObject:[NSValue valueWithCGPoint:[finger locationInView:[self superview]]] 
                 forKey:@"startDragLocation"];
        [dict setObject:[NSValue valueWithCGPoint:[finger locationInView:[self superview]]] 
                 forKey:@"lastStopPosition"];
    }else if(finger.state == UIGestureRecognizerStateChanged) {
        //如果是item，则判断是不是已经越界
        if ((self.frame.origin.x > 0-self.frame.size.width*0.5)
             &&(self.frame.origin.y > 0-self.frame.size.height*0.5)
             &&(self.frame.origin.x < ([self superview].frame.size.width-self.frame.size.width*0.5))
             &&(self.frame.origin.y) < ([self superview].frame.size.height-self.frame.size.height*0.5))
             
        {
            NSMutableDictionary *dict = (NSMutableDictionary*) (void*) self.tag;
            CGPoint lastStopPosition = [[dict valueForKey:@"lastStopPosition"] CGPointValue];
            
            CGPoint stopLocation = [finger locationInView:[self superview]];
            [dict setObject:[NSValue valueWithCGPoint:[finger locationInView:[self superview]]] 
                     forKey:@"lastStopPosition"];
            
            CGPoint startDragLocation = [[dict valueForKey:@"startDragLocation"] CGPointValue];
            CGPoint startDragOffset = [[dict valueForKey:@"startDragOffset"] CGPointValue];
            CGFloat dx = stopLocation.x - startDragLocation.x;
            CGFloat dy = stopLocation.y - startDragLocation.y;        
            
            //添加
            CGFloat moveDif_x=stopLocation.x-lastStopPosition.x;
            CGFloat moveDif_y=stopLocation.y-lastStopPosition.y;
            
            //   CGFloat distance = sqrt(dx*dx + dy*dy );
            CGRect dragFrame = self.frame;
            
            
            CGSize selfViewSize = [self superview].frame.size;
            if (!UIDeviceOrientationIsPortrait([[UIApplication sharedApplication] statusBarOrientation])) {
                selfViewSize = CGSizeMake(selfViewSize.height,selfViewSize.width);
            }
            
            selfViewSize.width  -= dragFrame.size.width;
            selfViewSize.height -= dragFrame.size.height;
            
            dragFrame.origin.x = MIN(selfViewSize.width, MAX(0,startDragOffset.x+dx));
            dragFrame.origin.y = MIN(selfViewSize.height,MAX(0,startDragOffset.y+dy
                                                             ));
            self.center=CGPointMake(self.center.x+moveDif_x,self.center.y+moveDif_y);
        }else{
            [self rePutToRightPlace];
        }
    }
    else if (finger.state == UIGestureRecognizerStateEnded) {
        [self rePutToRightPlace];
        //通知已经完成拖拽，是否需要更新背景拖拽
        [dict removeObjectForKey:@"startDragLocation"];
        [dict removeObjectForKey:@"startDragOffset"];
    }
}

- (void) rePutToRightPlace{
    if (self.frame.origin.x < -self.frame.size.width * 0.5) {
        self.center = CGPointMake(0+self.frame.size.width, self.frame.origin.y);
    }
    if (self.frame.origin.y < -self.frame.size.height * 0.5) {
        self.center = CGPointMake(0+self.frame.origin.x, self.frame.size.height);
    }
    if (self.frame.origin.x > ([self superview].frame.size.width-self.frame.size.width *0.5)) {
        self.center = CGPointMake([self superview].frame.size.width-self.frame.size.width, self.frame.origin.y);
    }
    if (self.frame.origin.y > ([self superview].frame.size.height-self.frame.size.height *0.5)) {
        self.center = CGPointMake(0+self.frame.origin.x, [self superview].frame.size.height-self.frame.size.height);
    }
}

- (NSMutableDictionary *) dictForViewGuestures{
    NSMutableDictionary *dict = (NSMutableDictionary*) (void*)self.tag;
    if (!dict) {
        dict = [[NSMutableDictionary dictionary ] retain];
        self.tag = (NSInteger) (void *) dict;
    }
    return dict;
}

- (void) setDelegate:(id)delegate{
    _delegate = delegate;
}

- (void) zoomWithValue:(float)value{
    CGFloat tarScale;
    CGFloat currentScale = [[self.layer valueForKeyPath:@"transform.scale"] floatValue];
    if (currentScale > 0) {
        tarScale = currentScale;    
    }else{
        tarScale = 0 - currentScale;
    }
    if (((tarScale > 1.5)&&(value > 1))||((tarScale < 0.5)&&(value < 1))) {
        
    }else{
        CGFloat newValue = value;
        CGAffineTransform transform = CGAffineTransformScale(self.transform, newValue, newValue);
        self.transform = transform;
    }
}

- (void)dealloc{
    
    self.image = nil;   
    [_imageName release];
    [super dealloc];
}
@end
