#import "SQLiteOperation.h"

@implementation SQLiteOperation

//implement this interface as singleton
#pragma mark Singleton definition
GTMOBJECT_SINGLETON_BOILERPLATE(SQLiteOperation, sharedSQLOpt)

#pragma mark -
#pragma mark Constructor and destructor
- (id) init
{
    NSLog(@"sqlite init!");
    
    self = [super init];
    BOOL success;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSError *error;
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *writableDBPath = [documentsDirectory stringByAppendingPathComponent:@"content.sqlite"];
    success = [fileManager fileExistsAtPath:writableDBPath];
	[self getPath];
    
    if (success) {
        if ([self openDB]) {
            return self;
        }
    }
    
    // The writable database does not exist, so copy the default to the appropriate location.
    NSString *defaultDBPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"content.sqlite"];
    success = [fileManager copyItemAtPath:defaultDBPath toPath:writableDBPath error:&error];
    if (!success) {
        NSAssert1(0, @"Failed to create writable database file with message '%@'.", [error localizedDescription]);
    }
    
    [self openDB];
    
    return self;
}

- (BOOL)openDB
{
    if (sqlite3_open([path UTF8String], &database) == SQLITE_OK) {
        return YES;
    }
    
    else {
        sqlite3_close(database);
        NSAssert1(0, @"Failed to open database with message '%s'.", sqlite3_errmsg(database));
        return NO;
    }
}

- (void)dealloc{
    sqlite3_close(database);
	[path release];
    [super dealloc];
}

- (void)getPath{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    path = [documentsDirectory stringByAppendingPathComponent:@"content.sqlite"];
}


- (NSMutableArray *)selectData:(NSString *)sql resultColumns:(int)col {
	NSMutableArray *returnArray = [[[NSMutableArray alloc] init] autorelease];//所有记录
    sqlite3_stmt *statement = nil;
    if (sqlite3_prepare_v2(database, [sql UTF8String], -1, &statement, NULL) == SQLITE_OK) {
        while (sqlite3_step(statement) == SQLITE_ROW) {
            NSMutableArray *row = [[NSMutableArray alloc] init];//一条记录
            for(int i=0; i<col; i++){
                [row addObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, i)]];
            }
            [returnArray addObject:row];
            [row release];
        }
    }else {
        NSLog(@"Error: failed to prepare");
    }
    
    sqlite3_finalize(statement);
	return returnArray;
}

#pragma mark 增，删，改数据库
- (BOOL)dealData:(NSString *)sql paramArray:(NSArray *)param {
    sqlite3_stmt *statement = nil;
    int success = sqlite3_prepare_v2(database, [sql UTF8String], -1, &statement, NULL);
	if (success != SQLITE_OK) {
        NSLog(@"Error: failed to prepare");
		return NO;
    }
	//绑定参数
	NSInteger max = [param count];
	for (int i=0; i<max; i++) {
		NSString *temp = [param objectAtIndex:i];
		sqlite3_bind_text(statement, i+1, [temp UTF8String], -1, SQLITE_TRANSIENT);
	}
    
	success = sqlite3_step(statement);
    sqlite3_finalize(statement);
	
    if (success == SQLITE_ERROR) {
		NSLog(@"Error: failed to insert into the database");
		return NO;
	}
	
	return TRUE;
}

- (BOOL) insertTbSaveBk:(NSString *)sql paramArray:(NSArray *)param{
    sqlite3_stmt *statement = nil;
    int success = sqlite3_prepare_v2(database, [sql UTF8String], -1, &statement, NULL);
	if (success != SQLITE_OK) {
        NSLog(@"Error: failed to prepare");
		return NO;
    }
    
    NSInteger max = [param count];
	for (int i=0; i<max; i++) {
		sqlite3_bind_text(statement, i+1, [[param objectAtIndex:i] UTF8String], -1, SQLITE_TRANSIENT);
    }
    
	success = sqlite3_step(statement);
    sqlite3_finalize(statement);
	
    if (success == SQLITE_ERROR) {
		NSLog(@"Error: failed to insert into the database");
		return NO;
	}
	
	return TRUE;
}

- (BOOL) insertTbSaveItem:(NSMutableDictionary *)dict withBkIdInt:(int)bkid{
    NSString* statement;
    NSString * sql = @"insert into tb_saveItem values(?,?,?,?,?,?,?,?,?,?,?,?,?)";
    statement = @"BEGIN EXCLUSIVE TRANSACTION";
    sqlite3_stmt *beginStatement;
    if (sqlite3_prepare_v2(database, [statement UTF8String], -1, &beginStatement, NULL) != SQLITE_OK) {
        printf("db error: %s\n", sqlite3_errmsg(database)); 
        return FALSE;
    }
    if (sqlite3_step(beginStatement) != SQLITE_DONE) {
        sqlite3_finalize(beginStatement);
        printf("db error: %s\n", sqlite3_errmsg(database)); 
        return FALSE;
    }
    
    
    sqlite3_stmt *compiledStatement = nil;
    int success = sqlite3_prepare_v2(database, [sql UTF8String], -1, &compiledStatement, NULL);
    
    if (success != SQLITE_OK) {
        //NSLog(@"Error: failed to prepare");
        printf("Error: failed to prepare");
		return NO;
    }
    
    NSArray * keys = [dict allKeys];    
    for(int i = 0; i < [keys count]; i++)
    {
        UIImageViewItemView * v = [dict objectForKey:[keys objectAtIndex:i]];
        
        NSString * itemId  = [keys objectAtIndex:i];
        NSString * itemBk = [NSString stringWithFormat:@"%d",bkid];
        NSString * itemName = v.imageName;
        NSString * itemCenter_x = [NSString stringWithFormat:@"%f",v.center.x];
        NSString * itemCenter_y = [NSString stringWithFormat:@"%f",v.center.y];
        NSString * itemTrans_a  = [NSString stringWithFormat:@"%f",v.transform.a];
        NSString * itemTrans_b  = [NSString stringWithFormat:@"%f",v.transform.b];
        NSString * itemTrans_c  = [NSString stringWithFormat:@"%f",v.transform.c];
        NSString * itemTrans_d  = [NSString stringWithFormat:@"%f",v.transform.d];
        NSString * itemTrans_tx = [NSString stringWithFormat:@"%f",v.transform.tx];
        NSString * itemTrans_ty = [NSString stringWithFormat:@"%f",v.transform.ty];
        NSString * itemOther = @"0";
        NSString * itemValue    = [NSString stringWithFormat:@"%d",v.pickViewCoin];  
        
        sqlite3_bind_text(  compiledStatement, 1, [itemId UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 2, [itemBk UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 3, [itemName UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 4, [itemCenter_x UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 5, [itemCenter_y UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 6, [itemTrans_a UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 7, [itemTrans_b UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 8, [itemTrans_c UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 9, [itemTrans_d UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 10, [itemTrans_tx UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 11, [itemTrans_ty UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 12, [itemOther UTF8String], -1, SQLITE_TRANSIENT);
        sqlite3_bind_text(  compiledStatement, 13, [itemValue UTF8String], -1, SQLITE_TRANSIENT);
        while(YES){
            NSInteger result = sqlite3_step(compiledStatement);
            if(result == SQLITE_DONE){
                break;
            }
            else if(result != SQLITE_BUSY){
                printf("insertMutiPerson db error: %s\n", sqlite3_errmsg(database)); 
                break;
            }
        }
        sqlite3_reset(compiledStatement);
    }
    
    statement = @"COMMIT TRANSACTION";
    sqlite3_stmt *commitStatement;
    if (sqlite3_prepare_v2(database, [statement UTF8String], -1, &commitStatement, NULL) != SQLITE_OK) {
        printf("insertMutiPerson2 db error: %s\n", sqlite3_errmsg(database)); 
    }
    if (sqlite3_step(commitStatement) != SQLITE_DONE) {
        printf("insertMutiPerson3 db error: %s\n", sqlite3_errmsg(database)); 
    }
    
    sqlite3_finalize(beginStatement);
    sqlite3_finalize(compiledStatement);
    sqlite3_finalize(commitStatement);
    
    return YES;
}

- (NSInteger) getSelectTopIdFromTbSaveBK{
    const char *selectSql = "select max(bKId) from tb_saveBK"; 
    sqlite3_stmt *statement = nil;
    int _id = 0;
    if (sqlite3_prepare_v2(database, selectSql, -1, &statement, nil)==SQLITE_OK)  
    {
        while (sqlite3_step(statement) == SQLITE_ROW) {
            _id = sqlite3_column_int(statement, 0); 
        }
    }else   
    {  
        _id = 1;
    }
    return _id;
}

-(BOOL)createNewTableWithName:(NSString *)sql
{
    NSArray *array=NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *filePath=[array objectAtIndex:0];
    
    filePath =[filePath stringByAppendingPathComponent:@"content.sqlite"];
    
    NSFileManager *manager=[NSFileManager defaultManager];
    
    BOOL success = NO;
    if ([manager fileExistsAtPath:filePath]) 
    {
        success =YES;
    }
    if (!success) 
    {
        NSString *path2=[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"content.sqlite"];
        success =[manager copyItemAtPath:path2 toPath:filePath error:nil];
    }
    sqlite3_stmt *createStmt = nil;
   
    if (sqlite3_open([filePath UTF8String], &database) == SQLITE_OK) {
        if (createStmt == nil) {
            if (sqlite3_prepare_v2(database, [sql UTF8String], -1, &createStmt, NULL) != SQLITE_OK) {
                return NO;
            }
            sqlite3_exec(database, [sql UTF8String], NULL, NULL, NULL);
            return YES;
        }
    }
    return YES;
}

@end